﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using SingularSys.Jep;
using SingularSys.Jep.Types;
using SingularSys.Jep.Functions;


namespace JepWindowExample
{
    /// <summary>
    /// Interaction logic for JepExampleWindow.xaml - A simple example using Jep.Net
    /// </summary>
    /// <remarks>
    /// This is a simple demo application to show how easy it is to plug 
    /// mathematical expression parsing and evaluation into an application
    /// using Jep.Net. The variable example in particular also gives a 
    /// great sense of Jep.Net's extremely fast parsing and evaluation.
    /// 
    /// For this example we have added the "add" function to our JepInstances,
    /// which has the following syntax "add(1,2,3)".
    /// </remarks>
    public partial class JepExampleWindow : Window
    {
        // One JepInstance for each tab (so variables don't get mixed up and values mistakingly changed)
        private JepInstance j = new JepInstance();
        private JepInstance jVar = new JepInstance();
        internal JepInstance jPlot = new JepInstance();

        // Graph elements for plotter tab
        private Polyline func = new Polyline();
        private Line xAxis = new Line();
        private Line yAxis = new Line();


        /// <summary>
        /// Initialize components, including JepInstances
        /// </summary>
        public JepExampleWindow()
        {
            j.FunTab.AddFunction("add", new Add());
            jVar.FunTab.AddFunction("add", new Add());
            InitializeComponent();
            jPlot.AllowUndeclared = false;
            jPlot.VarTab.AddVariable("x", new JepDouble(0));
        }

        private void JepExample_Loaded(object sender, RoutedEventArgs e)
        {
            this.plotCanvas.Init(this);
        }

        /*
         * ------------------------------------
         * - Methods for Simple Evaluator Tab -
         * ------------------------------------
         */ 
        /// <summary>
        /// If "Evaluate" button is clicked, grab the expression, parse and evaluate it.
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void evalbutton_Click(object sender, RoutedEventArgs e)
        {
            result.Content = ParseAndEval(exprSource.Text);
        }

        /// <summary>
        /// Sample was chosen. Set the expression content and call ParseAndEval
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void listView1_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            String expr = ((String)((ListViewItem)e.AddedItems[0]).Content);
            exprSource.Text = expr;

            result.Content = ParseAndEval(expr);
        }

        /// <summary>
        /// Parse and Evaluate an expression
        /// </summary>
        /// <param name="expr">The expression to evaluate</param>
        /// <returns>The result of the evaluation or error message if there is an exception</returns>
        private Object ParseAndEval(string expr)
        {
            Object res = null;
            // Try to Parse and Evaluate the expression
            try
            {
                j.Parse(expr);
                res = j.Evaluate();
            }
            catch (Exception err)
            {
                // If there is an error, then the result is the error message
                res = err.Message;
            }
            return res;
        }

        private void exprSource_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (result != null)
                result.Content = ParseAndEval(exprSource.Text);
        }

        /*
         * ------------------------------------
         * -     Methods for Variable Tab     -
         * ------------------------------------
         */
        /// <summary>
        /// If "Evaluate" button is clicked call off to VarParseAndEval
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void varEvalbutton_Click(object sender, RoutedEventArgs e)
        {
            VarParseAndEval();
        }

        /// <summary>
        /// Grab the variable values and expression. Parse and evaluate and output results.
        /// </summary>
        private void VarParseAndEval()
        {
            double x2digits = Math.Round(xVal.Value, 2);
            double y2digits = Math.Round(yVal.Value, 2);
            xValLabel.Content = "x=" + x2digits; 
            yValLabel.Content = "y=" + y2digits; 
            try
            {
                jVar.VarTab.AddVariable("x", new JepDouble(x2digits));
                jVar.VarTab.AddVariable("y", new JepDouble(y2digits));
                jVar.ReinitializeComponents();
                jVar.Parse(varExpr.Text);
                varResult.Content = jVar.Evaluate();
            }
            catch (Exception err)
            {
                varResult.Content = err.Message;
            }
        }

        /// <summary>
        /// Value changed: add new variable value to JepInstance and re-evaluate expression
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void slider1_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            VarParseAndEval();
        }

        /// <summary>
        /// Value changed: add new variable value to JepInstance and re-evaluate expression
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void yVal_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            VarParseAndEval();
        }

        /// <summary>
        /// Sample was chosen: set expression content and call VarParseAndEval
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void varExprList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            String expr = ((String)((ListViewItem)e.AddedItems[0]).Content);
            varExpr.Text = expr;
            VarParseAndEval();
        }


        private void varExpr_TextChanged(object sender, TextChangedEventArgs e)
        {
            if ((xVal != null) && (yVal != null))
                VarParseAndEval();
        }

        /*
         * ------------------------------------
         * -     Methods for Plotter Tab      -
         * ------------------------------------
         */
        
        /// <summary>
        /// Clear the canvas components and redraw the axes
        /// </summary>
        //public void DrawAxes()
        //{
        //    plotCanvas.Children.Clear();
        //    plotCanvas.Children.Add(border1);

        //    // Draw axes
        //    xAxis.X1 = 0;
        //    xAxis.X2 = plotCanvas.ActualWidth;
        //    xAxis.Y1 = plotCanvas.ActualHeight / 2;
        //    xAxis.Y2 = plotCanvas.ActualHeight / 2;
        //    xAxis.Stroke = System.Windows.Media.Brushes.Gray;
        //    xAxis.StrokeThickness = 0.5;
        //    plotCanvas.Children.Add(xAxis);

        //    yAxis.X1 = plotCanvas.ActualWidth / 2;
        //    yAxis.X2 = plotCanvas.ActualWidth / 2;
        //    yAxis.Y1 = 0;
        //    yAxis.Y2 = plotCanvas.ActualHeight;
        //    yAxis.Stroke = System.Windows.Media.Brushes.Gray;
        //    yAxis.StrokeThickness = 1;
        //    plotCanvas.Children.Add(yAxis);

        //    // Draw x-axis subdivisions
        //    for (double x = 0; x < (plotCanvas.ActualWidth /2) / xScale.Value; x += 1)
        //    {
        //        Line xdiv = new Line();
        //        xdiv.X1 = x * xScale.Value + (plotCanvas.ActualWidth / 2);
        //        xdiv.X2 = xdiv.X1;
        //        xdiv.Y1 = plotCanvas.ActualHeight / 2 - 3;
        //        xdiv.Y2 = plotCanvas.ActualHeight / 2 + 3;
        //        xdiv.Stroke = System.Windows.Media.Brushes.Gray;
        //        xdiv.StrokeThickness = 0.5;
        //        plotCanvas.Children.Add(xdiv);
        //    }
        //    for (double x = 0; x > -(plotCanvas.ActualWidth / 2) / xScale.Value; x -= 1)
        //    {
        //        Line xdiv = new Line();
        //        xdiv.X1 = x * xScale.Value + (plotCanvas.ActualWidth / 2);
        //        xdiv.X2 = xdiv.X1;
        //        xdiv.Y1 = plotCanvas.ActualHeight / 2 - 3;
        //        xdiv.Y2 = plotCanvas.ActualHeight / 2 + 3;
        //        xdiv.Stroke = System.Windows.Media.Brushes.Gray;
        //        xdiv.StrokeThickness = 0.5;
        //        plotCanvas.Children.Add(xdiv);
        //    }

        //    // Draw y-axis subdivisions
        //    for (double y = 0; y < (plotCanvas.ActualHeight / 2) / yScale.Value; y += 1)
        //    {
        //        Line ydiv = new Line();
        //        ydiv.X1 = (plotCanvas.ActualWidth / 2) -3;
        //        ydiv.X2 = (plotCanvas.ActualWidth / 2) + 3;
        //        ydiv.Y1 = y * yScale.Value + (plotCanvas.ActualHeight / 2);
        //        ydiv.Y2 = ydiv.Y1;
        //        ydiv.Stroke = System.Windows.Media.Brushes.Gray;
        //        ydiv.StrokeThickness = 0.5;
        //        plotCanvas.Children.Add(ydiv);
        //    }
        //    for (double y = 0; y > -(plotCanvas.ActualHeight / 2) / yScale.Value; y -= 1)
        //    {
        //        Line ydiv = new Line();
        //        ydiv.X1 = (plotCanvas.ActualWidth / 2) - 3;
        //        ydiv.X2 = (plotCanvas.ActualWidth / 2) + 3;
        //        ydiv.Y1 = y * yScale.Value + (plotCanvas.ActualHeight / 2);
        //        ydiv.Y2 = ydiv.Y1;
        //        ydiv.Stroke = System.Windows.Media.Brushes.Gray;
        //        ydiv.StrokeThickness = 0.5;
        //        plotCanvas.Children.Add(ydiv);
        //    }
        //}

        ///// <summary>
        ///// Take the contents from the expression field and iterate over x values
        ///// to calculate y-values.
        ///// </summary>
        //public void DrawFunction()
        //{
        //    try
        //    {
        //        jPlot.Parse(plotSource.Text);
        //    }
        //    catch (Exception) 
        //    {
        //        plotSource.Background = new SolidColorBrush(Colors.Salmon);
        //        return;
        //    }

        //    plotSource.Background = SystemColors.WindowBrush;

        //    PointCollection pc = new PointCollection();
        //    Double y = 0.0;
        //    for (double x = ((-plotCanvas.ActualWidth / 2) / xScale.Value); 
        //                x < ((plotCanvas.ActualWidth / 2) / xScale.Value); 
        //                x += 0.01)
        //    {
        //        Point p = new Point();
        //        p.X = (x * xScale.Value) + (plotCanvas.ActualWidth / 2);
        //        if (Math.Abs(x - 0.0000000) < 1e-06)
        //            x = 0.0000000001;
        //        jPlot.VarTab.AddVariable("x", new JepDouble(x));
        //        jPlot.VarTab.Init(jPlot);
        //        try
        //        {
        //            y = ((JepDouble)jPlot.Evaluate()).DoubleValue;
        //        }
        //        catch (Exception e)
        //        {
        //            y = 0; 
        //        }
        //        p.Y = (plotCanvas.ActualHeight / 2) - y * yScale.Value;
        //        pc.Add(p);
        //    }


        //    func.Points = pc;
        //    func.Stroke = Brushes.Blue;
        //    func.StrokeThickness = 0.5;
        //    plotCanvas.Children.Add(func);
        //    plotCanvas.Clip = new RectangleGeometry(new Rect(0, 0, plotCanvas.ActualWidth, plotCanvas.ActualHeight));
        //}

        /// <summary>
        /// Sample selected - set the function and redraw graph
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void plotList_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            String expr = ((String)((ListViewItem)e.AddedItems[0]).Content);
            plotSource.Text = expr;

            plotCanvas.InvalidateVisual();
        }

        /// <summary>
        /// Scale value(s) changed - redraw function
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void Scale_ValueChanged(object sender, RoutedPropertyChangedEventArgs<double> e)
        {
            if (plotCanvas != null)
            {
                bool locked = (bool)scaleLock.IsChecked;
                if (locked && (sender.Equals(xScale)))
                    yScale.Value = xScale.Value;
                else if (locked)
                    xScale.Value = yScale.Value;
                plotCanvas.InvalidateVisual();
            }
        }

        /// <summary>
        /// Scale lock selected - lock x and y sliders
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void scaleLock_Checked(object sender, RoutedEventArgs e)
        {
            yScale.Value = xScale.Value;
        }


        private void plotSource_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (plotCanvas != null)
            {
                plotCanvas.InvalidateVisual();
            }
        }

        private void plotCanvas_Loaded(object sender, RoutedEventArgs e)
        {
            plotCanvas.Init(this);
            plotCanvas.InvalidateVisual();
        }

        private void JepExample_SizeChanged(object sender, SizeChangedEventArgs e)
        {
            plotCanvas.InvalidateVisual();
        }
    }

    public class PlotCanvas : Canvas
    {
        JepExampleWindow parWin;
        JepInstance jPlot;

        // Graph elements for plotter tab
        private Polyline func = new Polyline();
        private Line xAxis = new Line();
        private Line yAxis = new Line();

        public void Init(JepExampleWindow parentWin)
        {
            parWin = parentWin;
            jPlot = parWin.jPlot;
        }

        protected override void OnRender(DrawingContext dc)
        {
            base.OnRender(dc);

            if (parWin != null)
            {
                // --------- Draw axes
                Pen myPen = new Pen(System.Windows.Media.Brushes.Gray, 0.5);
                dc.DrawLine(myPen, new Point(0, this.ActualHeight / 2), new Point(this.ActualWidth, this.ActualHeight / 2));
                dc.DrawLine(myPen, new Point(this.ActualWidth / 2, 0), new Point(this.ActualWidth / 2, this.ActualHeight));

                // Draw x-axis subdivisions
                for (double x = 0; x < (this.ActualWidth / 2) / parWin.xScale.Value; x += 1)
                {
                    dc.DrawLine(myPen,
                        new Point(x * parWin.xScale.Value + (this.ActualWidth / 2), this.ActualHeight / 2 - 3),
                        new Point(x * parWin.xScale.Value + (this.ActualWidth / 2), this.ActualHeight / 2 + 3));
                }
                for (double x = 0; x > -(this.ActualWidth / 2) / parWin.xScale.Value; x -= 1)
                {
                    dc.DrawLine(myPen,
                        new Point(x * parWin.xScale.Value + (this.ActualWidth / 2), this.ActualHeight / 2 - 3),
                        new Point(x * parWin.xScale.Value + (this.ActualWidth / 2), this.ActualHeight / 2 + 3));
                }

                // Draw y-axis subdivisions
                Double y = 0.0;
                for (y = 0; y < (this.ActualHeight / 2) / parWin.yScale.Value; y += 1)
                {
                    dc.DrawLine(myPen,
                        new Point((this.ActualWidth / 2) - 3, y * parWin.yScale.Value + (this.ActualHeight / 2)),
                        new Point((this.ActualWidth / 2) + 3, y * parWin.yScale.Value + (this.ActualHeight / 2)));
                }

                for (y = 0; y > -(this.ActualHeight / 2) / parWin.yScale.Value; y -= 1)
                {
                    dc.DrawLine(myPen,
                        new Point((this.ActualWidth / 2) - 3, y * parWin.yScale.Value + (this.ActualHeight / 2)),
                        new Point((this.ActualWidth / 2) + 3, y * parWin.yScale.Value + (this.ActualHeight / 2)));
                }

                // ------------- Draw Function
                try
                {
                    parWin.jPlot.Parse(parWin.plotSource.Text);
                }
                catch (Exception)
                {
                    parWin.plotSource.Background = new SolidColorBrush(Colors.Salmon);
                    return;
                }

                parWin.plotSource.Background = SystemColors.WindowBrush;

                PolyLineSegment myLineSegs = new PolyLineSegment();
                JepDouble xDouble = new JepDouble(0);
                jPlot.VarTab.AddVariable("x", xDouble);
                Point p = new Point();
                for (double x = ((-this.ActualWidth / 2) / parWin.xScale.Value);
                            x < ((this.ActualWidth / 2) / parWin.xScale.Value);
                            x += 0.01)
                {
                    p.X = (x * parWin.xScale.Value) + (this.ActualWidth / 2);
                    if (Math.Abs(x - 0.0000000) < 1e-06)
                        x = 0.0000000001;
                    xDouble.Value = x;
                    //jPlot.VarTab.Init(jPlot);
                    try
                    {
                        y = ((JepDouble)jPlot.Evaluate()).DoubleValue;
                    }
                    catch (Exception)
                    {
                        y = 0;
                    }
                    p.Y = (this.ActualHeight / 2) - y * parWin.yScale.Value;
                    myLineSegs.Points.Add(p);
                }

                PathFigure myPathFigure = new PathFigure();
                myPathFigure.Segments.Add(myLineSegs);

                PathGeometry myPathGeometry = new PathGeometry();
                myPathGeometry.Figures.Add(myPathFigure);

                Pen funcPen = new Pen(Brushes.Blue, 1);
                dc.DrawGeometry(null, funcPen, myPathGeometry);
            }
        }

        protected override void OnRenderSizeChanged(SizeChangedInfo sizeInfo)
        {
            base.OnRenderSizeChanged(sizeInfo);
            this.Clip = new RectangleGeometry(new Rect(0, 0, this.ActualWidth, this.ActualHeight));
        }

    }

}
