﻿using System;
using System.Collections.Generic;
using System.Text;

using NUnit.Framework;

using SingularSys.Jep.Functions;
using SingularSys.Jep.Parser;
using SingularSys.Jep.Standard;
using SingularSys.Jep;
using SingularSys.Jep.Types;
using SingularSys.Jep.ConfigurableParser;
using SingularSys.Jep.Reals;

namespace ConsoleApplication1
{
    class Program
    {
        Boolean myTrue = true;
        Boolean myFalse = false;

        static void Main(string[] args)
        {
            String expr = "add(1,2,3,4,5,6)";

            JepInstance j = new JepInstance();
            j.FunTab.AddFunction("add", new Add());
            //j.Parser = new StandardConfigurableParser();
            //j.Evaluator = new StandardEvaluator();
            j.Parse(expr);
            Object result = j.Evaluate();
            Console.WriteLine("{0} evaluates as {1}", expr, result);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);
        }

        public const bool PRINT_RESULTS = true;
        /** The parser */
        protected JepInstance jep;

        /**
         * Sets up the parser.
         */
        [SetUp]
        public void Init()
        {
            // Set up the parser
            jep = new JepInstance();
            jep.ImplicitMul = true;
            //jep.AddStandardFunctions();
            jep.AddStandardConstants();
            //jep.AddComplex();
            //jep.SetTraverse(false);
        }

        /*------------------- utility functions -------------------------------*/

        /**
         * Prints a header with the name of the test as specified in str.
         */
        protected void printTestHeader(String str)
        {
            System.Console.WriteLine("\n\n------------------------------------------------------------------------");
            System.Console.WriteLine("Running \"" + str + "\"\n");
        }

        /**
         * Test result j.evaluate(j.parse(expr))
         * @param expr the expression to parse and evaluate
         * @param expected result expected
         * @throws Exception
         */
        protected void valueTest(String expr, Object expected)
        {
            INode node = jep.Parse(expr);
            Object res = jep.Evaluate(node);

            myAssertEquals(expr, expected, res);
        }

        /**
         * Calculate the value of an expression.
         * @param node
         * @throws Exception
         */
        protected Object calcValue(INode node)
        {
            Object res = jep.Evaluate(node);
            return res;
        }

        /**
         * Calculate the value of an expression.
         * @param expr
         * @throws Exception
         */
        protected Object calcValue(String expr)
        {
            INode node = jep.Parse(expr);
            Object res = calcValue(node);
            return res;
        }

        protected void myAssertEquals(String msg, Object expected, Object actual)
        {
            if (PRINT_RESULTS && !actual.Equals(expected))
                System.Console.WriteLine("Error: '" + msg + "' is '" + actual + "' should be '" + expected + "'");
            Assert.AreEqual(expected, actual, "<" + msg + ">");
            if (PRINT_RESULTS)
                System.Console.WriteLine("Success: Value of \"" + msg + "\" is " + actual + "");
        }

        public void myAssertNaN(String msg, Object actual)
        {
            if (actual is Double)
            {
                if (Double.IsNaN(((Double)actual)))
                {
                    System.Console.WriteLine("Success: Value of \"" + msg + "\" is " + actual + "");
                }
                else
                {
                    System.Console.WriteLine("Error: \"" + msg + "\" is '" + actual + "' should be NaN");
                    Assert.Fail("<" + msg + "> is " + actual + " should be NaN");
                }
            }
            else
            {
                System.Console.WriteLine("Error: '" + msg + "' is '" + actual + "' should be 'NaN'");
                Assert.Fail("<" + msg + ">");
            }
        }

        /**
         * Test whether an expression evaluates to NaN
         * @param expr
         * @throws Exception
         */
        public void valueTestNaN(String expr)
        {
            Object res = calcValue(expr);
            myAssertNaN(expr, res);
        }

        public void valueTestString(String expr, String expected)
        {
            Object res = calcValue(expr);
            myAssertEquals(expr, expected, res.ToString());
        }
        /** Test parse-evaluate with complex number and given tolerence.
         * 
         * @param expr
         * @param expected
         * @param tol
         * @throws Exception
         */
        public void complexValueTest(String expr, Complex expected, double tol)
        {
            Object res = calcValue(expr);
            if (expected.IsEqual((Complex)res, tol))
                System.Console.WriteLine("Success value of \"" + expr + "\" is " + res);
            else
            {
                System.Console.WriteLine("Error value of \"" + expr + "\" is " + res + " should be " + expected);
                Assert.Fail("<" + expr + "> expected: <" + expected + "> but was <" + res + ">");
            }
        }

        /**
         * Test values to within a given precision
         * 
         * @param expr
         *            expression
         * @param a
         *            the expected value
         * @param tol
         *            tolerance
         * @throws Exception
         */
        public void valueTest(String expr, double a, double tol)
        {
            Object res = calcValue(expr);
            if (res is Double)
            {
                double val = ((Double)res);
                if (Math.Abs(val - a) < tol)
                {
                    System.Console.WriteLine("Success value of \"" + expr + "\" is "
                            + res);
                }
                else
                {
                    System.Console.WriteLine("Error value of \"" + expr + "\" is " + res
                            + " should be " + a);
                    Assert.AreEqual(a, val, tol, expr);
                }
            }
            else
            {
                System.Console.WriteLine("Error value of \"" + expr + "\" is " + res
                        + " should be " + a);
                Assert.Fail("<" + expr + "> expected: <" + a + "> but was <" + res + ">");
            }
        }

        static public void nodeTest(INode n, Operator op)
        {
            Assert.IsTrue(n is ASTOpNode, "Node " + n.ToString() + "should have been an ASTOpNode");
            Assert.AreEqual(op, ((ASTOpNode)n).GetOperator());
        }

        static public void nodeTest(INode n, String name)
        {
            Assert.IsTrue(n is ASTFunNode, "Node " + n.ToString() + "should have been an ASTFunNode");
            Assert.AreEqual(name, ((ASTFunNode)n).GetName());
        }

        static public void nodeTest(INode n, Variable v)
        {
            Assert.IsTrue(n is ASTVarNode, "Node " + n.ToString() + "should have been an ASTVarNode");
            Assert.AreEqual(v, ((ASTVarNode)n).Var);
        }

        static public void nodeTest(INode n, Object v)
        {
            Assert.IsTrue(n is ASTConstant, "Node " + n.ToString() + "should have been an ASTConstant node");
            Assert.AreEqual(v, ((ASTConstant)n).Value);
        }

        public bool compareRecursive(INode node1, INode node2)
        {
            if (node1.JjtGetNumChildren() != node2.JjtGetNumChildren())
                return false;
            if (node1 is ASTConstant)
            {
                if (node2 is ASTConstant)
                {
                    return ((ASTConstant)node1).Value.Equals(((ASTConstant)node2).Value);
                }
                return false;
            }
            if (node1 is ASTVarNode)
            {
                if (node2 is ASTVarNode)
                {
                    return ((ASTVarNode)node1).GetName().Equals(((ASTVarNode)node2).GetName());
                }
                return false;
            }
            if (node1 is ASTOpNode)
            {
                if (node2 is ASTOpNode)
                {
                    if (!((ASTOpNode)node1).GetOperator().Equals(((ASTOpNode)node2).GetOperator()))
                        return false;
                    for (int i = 0; i < node1.JjtGetNumChildren(); ++i)
                    {
                        if (!compareRecursive(node1.JjtGetChild(i), node2.JjtGetChild(i)))
                            return false;
                    }
                    return true;
                }
            }
            if (node1 is ASTFunNode)
            {
                if (node2 is ASTFunNode)
                {
                    if (!((ASTFunNode)node1).GetName().Equals(((ASTFunNode)node2).GetName()))
                        return false;
                    if (((ASTFunNode)node1).GetPFMC() != (((ASTFunNode)node2).GetPFMC()))
                        return false;
                    for (int i = 0; i < node1.JjtGetNumChildren(); ++i)
                    {
                        if (!compareRecursive(node1.JjtGetChild(i), node2.JjtGetChild(i)))
                            return false;
                    }
                    return true;
                }
            }
            return false;
        }

        public void assertEqNodes(String s, INode expected, INode actual)
        {
            bool flag = compareRecursive(expected, actual);
            bool mode = jep.PrintVisitor.GetMode(PrintVisitor.FULL_BRACKET);
            jep.PrintVisitor.SetMode(PrintVisitor.FULL_BRACKET, true);
            if (!flag)
            {
                System.Console.Write("Parse: ");
                System.Console.WriteLine(s.Replace(">\\s+", ">").Replace("\\s+<", "<"));
                System.Console.WriteLine("actual:\t\t" + jep.ToString(actual));
                if (!flag)
                    System.Console.WriteLine("expected:\t" + jep.ToString(expected));
            }
            jep.PrintVisitor.SetMode(PrintVisitor.FULL_BRACKET, mode);
            Assert.IsTrue(flag, "expected: " + jep.ToString(expected) + " actual: " + jep.ToString(actual));
            //		System.Console.WriteLine("\nSuccess");
            //		System.Console.WriteLine(s.replaceAll(">\\s+",">").replaceAll("\\s+<","<"));
            //		jep.println(actual);
        }

        public void testNaN()
        {
            printTestHeader("Testing for NaN");
            double x = 1.0, y = 1.0;
            double z = x / y;
            if (z < 2.0)
                System.Console.WriteLine("1/0<2 is true");
            else
                System.Console.WriteLine("1/0<2 is false");

            jep.AddVariable("x", double.NaN);
            System.Console.WriteLine("x=NaN");
            valueTestNaN("ln(x)");
            valueTestNaN("log(x)");
            valueTestNaN("sin(x)");
            valueTestNaN("x+x");
            valueTest("x!=x", myTrue);
            valueTest("x==x", myFalse);

            jep.AddVariable("y", Double.NaN);
            INode n = jep.Parse("x+5");
            System.Console.WriteLine(calcValue(n));
            INode n2 = jep.Parse("y");
            System.Console.WriteLine(calcValue(n2));
            valueTest("x == x+5", myFalse);
            valueTest("x == 0/0", myFalse);
            valueTest("x == x", myFalse);
            valueTest("x == 0 * x", myFalse);
            valueTest("x == 5", myFalse);
            valueTest("x == y", myFalse);
            valueTest("y == y", myFalse);
            System.Console.WriteLine("Set x to Double(5)");
            jep.AddVariable("x", (Double)5);
            valueTest("x == x+5", myFalse);
            valueTest("x == x", myTrue);
        }
    }
}
